#include "cringbuffer.h"

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//- ---------------------------------------------------------------------------------
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
CRingBuffer::CRingBuffer(unsigned long size)
{ 
 Buffer=new unsigned char[size+1];
 Reset();
 Size=size;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//- ----------------------------------------------------------------------------------
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
CRingBuffer::~CRingBuffer()
{
 delete[](Buffer);
}
//- ------------------------------------------------------------

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//----------------------------------------------------------------------------------------------------
//    
//----------------------------------------------------------------------------------------------------
void CRingBuffer::PushByte(unsigned char b)
{
 Buffer[Head]=b;
 Head++;
 if (Head==Size) Head=0;
 if (Head==Tail) Tail++;
 if (Tail==Size) Tail=0;	
}
//----------------------------------------------------------------------------------------------------
//    
//----------------------------------------------------------------------------------------------------
bool CRingBuffer::PopByte(unsigned char *b)
{
 if (Head==Tail) return(false);
 *b=Buffer[Tail];
 Tail++;
 if (Tail==Size) Tail=0;
 return(true);	
}

//----------------------------------------------------------------------------------------------------
//     
//----------------------------------------------------------------------------------------------------
bool CRingBuffer::PopShort(unsigned short *s)
{
 if (PopByte((unsigned char*)(s)+0)==false) return(false);
 if (PopByte((unsigned char*)(s)+1)==false) return(false);
 return(true);	
}

//----------------------------------------------------------------------------------------------------
//     
//----------------------------------------------------------------------------------------------------
unsigned long CRingBuffer::GetDataSize(void) const
{
 if (Head<Tail) return(Size-Tail+Head);
 return(Head-Tail);
}
//----------------------------------------------------------------------------------------------------
//   
//----------------------------------------------------------------------------------------------------
unsigned long CRingBuffer::GetHeadPos(void) const
{
 return(Head);
}
//----------------------------------------------------------------------------------------------------
//   
//----------------------------------------------------------------------------------------------------
unsigned long CRingBuffer::GetTailPos(void) const
{
 return(Tail);
}
//----------------------------------------------------------------------------------------------------
//   
//----------------------------------------------------------------------------------------------------
unsigned long CRingBuffer::GetBufferSize(void) const
{
 return(Size);
}
//----------------------------------------------------------------------------------------------------
//   
//----------------------------------------------------------------------------------------------------
void CRingBuffer::Reset(void)
{
 Head=0;
 Tail=0;
}
//----------------------------------------------------------------------------------------------------
//   
//----------------------------------------------------------------------------------------------------
unsigned char CRingBuffer::GetByte(unsigned long pos) const
{
 if (pos>=Size) return(0);
 return(Buffer[pos]);
}
//----------------------------------------------------------------------------------------------------
//   
//----------------------------------------------------------------------------------------------------
void CRingBuffer::SetTailPos(unsigned long pos)
{
 if (pos>=Size) return;
 Tail=pos;
}
//----------------------------------------------------------------------------------------------------
//   
//----------------------------------------------------------------------------------------------------
void CRingBuffer::SetHeadPos(unsigned long pos)
{
 if (pos>=Size) return;
 Head=pos;
}

